/*
 * MILIB.H -- LIBMI structures and prototypes
 *
 *     Copyright c 1995 Illustra Information Technologies, Inc.
 *     All rights reserved.      See files/copyright.
 *
 */

#ifndef _MILIB_H_
#define _MILIB_H_

#ifndef _MITYPES_H_
#include "mitypes.h"
#endif /* _MITYPES_H_ */


/* define NULL if we don't have it yet */
#ifndef NULL
#define NULL	0
#endif /* NULL */


/*
 * ErrorRaise error levels.
 */
#define MI_MESSAGE	1
#define MI_NOTICE       MI_MESSAGE
#define MI_EXCEPTION	2
#define MI_WARN         MI_EXCEPTION
#define MI_SQL		3
#define MI_FATAL        4


/*
 * Client errors
 */
#define MI_LIB_BADARG	5	/* Bad arg to LIBDB function */
#define MI_LIB_USAGE	6	/* Bad LIBDB usage sequence  */
#define MI_LIB_INTERR	7	/* Internal error in LIBDB   */
#define MI_LIB_NOIMP	8	/* Library feature not implemented */
#define MI_LIB_DROPCONN	9	/* Connection to server lost */
#define MI_LIB_BADSERV	10	/* Can't connect to MI_SERVER */


/*
 * function result masks
 */
#define MI_INTERNAL_REP_RESULT	0x0001
#define MI_ALLOCATED_RESULT	0x0002
#define MI_NO_RESULT		0x0004
#define MI_NULL_RESULT		0x0008


/*
 * Asynchronous action possibilities
 */
typedef enum mi_async_action
{
    MI_AS_NONE,
    MI_AS_RESULT,
    MI_AS_ROW,
    MI_AS_ERROR,
    MI_AS_DEAD,
    MI_AS_UNKNOWN
} MI_ASYNC_ACTION;

/*
 * Cursor Actions
 */
typedef enum mi_cursor_action
{
    MI_CURSOR_NEXT,
    MI_CURSOR_PRIOR,
    MI_CURSOR_FIRST,
    MI_CURSOR_LAST,
    MI_CURSOR_ABSOLUTE,
    MI_CURSOR_RELATIVE,
    MI_CURSOR_OPEN,
    MI_CURSOR_CLOSE
} MI_CURSOR_ACTION;

/*
 * Client function types
 */
typedef enum mi_functype
{
    MI_C_FUNC,
    MI_VISUAL_BASIC_FUNC,
    MI_PASCAL_FUNC
} MI_FUNCTYPE;


/*
 * Arguments used for mi_server_connect()
 */
#define MI_USE_DEFAULT_PORT	(-1)	/* directive to use default port */


/*
 * Masks used for mi_exec()
 */
#define MI_QUERY_NORMAL		0x0000	/* string rep return values */
#define MI_QUERY_BINARY		0x0001	/* binary return values */


/*
 * Return values for mi_get_result()
 */
#define MI_ERROR		(mi_integer)(-1)
#define MI_NO_MORE_RESULTS	(mi_integer)0
#define	MI_ROWS			(mi_integer)1
#define MI_DML			(mi_integer)3
#define MI_DDL			(mi_integer)4
#define MI_COPY_OUTPUT		(mi_integer)5
#define MI_COPY_INPUT		(mi_integer)6
#define MI_FUNCTION_RESULTS	(mi_integer)7
#define MI_OK			0


/*
 * Return values for mi_value()
 */
#define MI_NORMAL_VALUE		0x0000
#define MI_NULL_VALUE		0x0001
#define MI_ROW_VALUE		0x0002

/* support for these is not implemented yet */
#define MI_PORTAL_VALUE		0x0004
#define MI_ARRAY_VALUE		0x0008
#define MI_LARGE_OBJECT_VALUE	0x0010
 

/*
 * masks for alerter status
 */
#define MI_ALERTER_FIRED        0x0001
#define MI_ALERTER_DROPPED      0x0002


/*
 * maximum number of arguments passed to a client function 
 */
#define MI_MAX_FUNC_ARGS	20


/*
 * Globally accessible yet opaque structures.
 */

typedef struct mi_connection	MI_CONNECTION;
typedef struct mi_statement	MI_STATEMENT;
typedef struct mi_save_set	MI_SAVE_SET;
typedef struct mi_row		MI_ROW;
typedef struct mi_row_desc	MI_ROW_DESC;
typedef struct mi_error_desc	MI_ERROR_DESC;


/* The following structures are completely public structures. */
typedef struct mi_connection_info
{
    char	MI_FAR *server_name;
    char	MI_FAR *server_host;
    mi_integer 	server_port;
    char	MI_FAR *language;
    char	MI_FAR *encoding;
} MI_CONNECTION_INFO;

typedef struct mi_database_info
{
    char	MI_FAR *database_name;
    char	MI_FAR *user_name;
    char	MI_FAR *password;
} MI_DATABASE_INFO;

typedef struct mi_parameter_info
{
    mi_integer callbacks_enabled;
    mi_integer pointer_checks_enabled;
} MI_PARAMETER_INFO;


/*
 * Public data structures.
 */

typedef enum mi_event_type
{
    MI_All_Events		= -1,	/* Special All Events flag */
    MI_Exception		= 0,	/* an exception, (server error) */
    MI_Alerter_Fire_Msg		= 1,	/* An alerter callback */
    MI_Delivery_Status_Msg	= 2,	/* A Delivery Status callback */
    MI_Query_Interrupt_Ack	= 3,	/* A Query Interrupt acknowledgement */
    MI_Client_Library_Error	= 4,	/* Client-side library errors */
    MI_Xact_State_Change	= 5,	/* Transaction State Change */
    MI_Print			= 6,	/* print a text string */
    MI_Request			= 7,	/* prompt for & get a user response */
    MI_Tape_Request		= 8	/* Tape Request callback */
} MI_EVENT_TYPE;

#define MI_N_CALLBACK_EVENTS	10

/* valid callback retcodes */
typedef enum mi_callback_status 
{ 
    MI_CB_CONTINUE = 0
} MI_CALLBACK_STATUS;

/* handle to a registered callback function */
typedef struct mi_callback_handle MI_CALLBACK_HANDLE;


/*
 * Function pointer types.
 */
typedef void MI_FAR *	(* MI_CALLFUNC_PTR) ();
typedef char MI_FAR *	(* MI_FUNC_PTR) ();
typedef void MI_FAR *	(MI_PROC_VACALLBACK * MI_C_FUNC_PTR) ();
typedef void MI_FAR *	(MI_PROC_CALLBACK * MI_PASCAL_FUNC_PTR) ();
typedef void (MI_PROC_CALLBACK * MI_VOID) ARGS((MI_EVENT_TYPE type,
						MI_CONNECTION MI_FAR *conn,
						void MI_FAR *cb_data,
						void MI_FAR *user_data));
typedef MI_CALLBACK_STATUS (MI_PROC_CALLBACK * MI_CALLBACK_FUNC)
					  ARGS((MI_EVENT_TYPE type, 
						MI_CONNECTION MI_FAR *conn, 
						void MI_FAR *cb_data, 
						void MI_FAR *user_data));
typedef mi_boolean      (* MI_BOOL_FUNC_PTR) ();
typedef mi_varlena MI_FAR *    (* MI_VARLENA_FUNC_PTR) ();


/*
 * The MI_Request callback argument descriptor
 */
typedef struct mi_request
{
    char   MI_FAR *prompt;	/* present this prompt to the user */
    char   MI_FAR *response;	/* place for null-termimated response */
    mi_integer  response_length;/* size of the response buffer */
} MI_REQUEST;

typedef void (MI_PROC_CALLBACK * MI_REQUEST_FUNC)
				ARGS((MI_EVENT_TYPE event,
				      mi_integer dbconn,
				      MI_REQUEST MI_FAR * reqdesc,
				      void MI_FAR * user_data));


/*--------------------------
 * Public function prototypes.
 *--------------------------
 */

EXTERNC_BEGIN

mi_integer MI_PROC_EXPORT
mi_add_callback ARGS((MI_EVENT_TYPE event_type,
		      MI_VOID func,
		      void MI_FAR *user_data));

mi_integer MI_PROC_EXPORT
mi_alert_nalerts ARGS((void MI_FAR *ae));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_alert_name ARGS((void MI_FAR *ae));

mi_integer MI_PROC_EXPORT
mi_alert_status ARGS((void MI_FAR *ae));

char MI_FAR * MI_PROC_EXPORT
mi_alloc ARGS((mi_integer len));

mi_integer MI_PROC_EXPORT
mi_binary_query ARGS((MI_CONNECTION MI_FAR *conn));

void MI_PROC_EXPORT
mi_cleanup ARGS((void));

mi_integer MI_PROC_EXPORT
mi_client ARGS((void));

mi_integer MI_PROC_EXPORT
mi_close ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_column_bound ARGS((MI_ROW_DESC MI_FAR *idesc,
		      mi_integer value_no));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_column_constructor_name ARGS((MI_ROW_DESC MI_FAR *idesc,
				 mi_integer value_no));
char MI_FAR * MI_PROC_EXPORT
mi_column_constructor_id ARGS((MI_ROW_DESC MI_FAR *idesc,
			       mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_count ARGS((MI_ROW_DESC MI_FAR *idesc));

mi_integer MI_PROC_EXPORT
mi_column_id ARGS((MI_ROW_DESC MI_FAR *idesc,
		   const MI_STRING MI_FAR *colname));

mi_integer MI_PROC_EXPORT
mi_column_is_arrayof ARGS((MI_ROW_DESC MI_FAR *idesc,
			   mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_is_composite ARGS((MI_ROW_DESC MI_FAR *idesc,
			     mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_is_ref ARGS((MI_ROW_DESC MI_FAR *idesc,
		       mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_is_setof ARGS((MI_ROW_DESC MI_FAR *idesc,
			 mi_integer value_no));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_column_name ARGS((MI_ROW_DESC MI_FAR *idesc,
		     mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_nullable ARGS((MI_ROW_DESC MI_FAR *idesc,
			 mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_parameter ARGS((MI_ROW_DESC MI_FAR *idesc,
			  mi_integer value_no));

char MI_FAR * MI_PROC_EXPORT
mi_column_subtype_id ARGS((MI_ROW_DESC MI_FAR *idesc,
			   mi_integer value_no));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_column_subtype_name ARGS((MI_ROW_DESC MI_FAR *idesc,
			     mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_type_align ARGS((MI_ROW_DESC MI_FAR *idesc,
			   mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_type_bound ARGS((MI_ROW_DESC MI_FAR *idesc,
			   mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_type_byvalue ARGS((MI_ROW_DESC MI_FAR *idesc,
			     mi_integer value_no));

char MI_FAR * MI_PROC_EXPORT
mi_column_type_id ARGS((MI_ROW_DESC MI_FAR *idesc,
			mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_type_length ARGS((MI_ROW_DESC MI_FAR *idesc,
			    mi_integer value_no));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_column_type_name ARGS((MI_ROW_DESC MI_FAR *idesc,
			  mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_column_type_parameter ARGS((MI_ROW_DESC MI_FAR *idesc,
			       mi_integer value_no));

mi_integer MI_PROC_EXPORT
mi_command_is_finished ARGS((MI_CONNECTION MI_FAR *conn));

MI_ROW_DESC MI_FAR * MI_PROC_EXPORT
mi_composite_column_desc ARGS((MI_ROW_DESC MI_FAR *idesc,
			       mi_integer value_no));

MI_CONNECTION MI_FAR * MI_PROC_EXPORT
mi_connect ARGS((MI_CONNECTION_INFO MI_FAR *cinfo,
		 MI_DATABASE_INFO MI_FAR *dinfo));

char MI_FAR * MI_PROC_EXPORT
mi_dalloc ARGS((mi_integer len,
		MI_MEMORY_DURATION duration));

mi_integer MI_PROC_VAEXPORT
mi_db_error_raise ARGS((MI_CONNECTION MI_FAR *conn,
			mi_integer msg_type,
			const char MI_FAR *msg,
			...));

void MI_PROC_EXPORT
mi_default_callback ARGS((MI_EVENT_TYPE type,
			  MI_CONNECTION MI_FAR *conn,
			  void MI_FAR *s1,
			  void MI_FAR *user_data));

mi_integer MI_PROC_EXPORT
mi_delete_callback ARGS((MI_EVENT_TYPE event_type));

mi_integer MI_PROC_EXPORT
mi_disable_callback ARGS((MI_CONNECTION *conn,
			  MI_EVENT_TYPE event_type,
			  MI_CALLBACK_HANDLE MI_FAR *handle));

mi_integer MI_PROC_EXPORT
mi_disable_callbacks ARGS((void));

mi_integer MI_PROC_EXPORT
mi_enable_callback ARGS((MI_CONNECTION *conn,
			 MI_EVENT_TYPE event_type,
			 MI_CALLBACK_HANDLE MI_FAR *handle));

mi_integer MI_PROC_EXPORT
mi_enable_callbacks ARGS((void));

void MI_PROC_EXPORT
mi_errmsg ARGS((void MI_FAR *estruct,
		char MI_FAR *buf,
		mi_integer buflen));

MI_ERROR_DESC MI_FAR * MI_PROC_EXPORT
mi_error_desc_copy ARGS((MI_ERROR_DESC MI_FAR *desc));

mi_integer MI_PROC_EXPORT
mi_error_desc_destroy ARGS((MI_ERROR_DESC MI_FAR *desc));

mi_integer MI_PROC_EXPORT
mi_error_desc_is_copy ARGS((MI_ERROR_DESC MI_FAR *desc));

mi_integer MI_PROC_EXPORT
mi_error_level ARGS((MI_ERROR_DESC MI_FAR *estruct));

mi_integer MI_PROC_EXPORT
mi_error_sql_code ARGS((MI_ERROR_DESC MI_FAR *estruct,
			char MI_FAR *buf,
			mi_integer buflen));

mi_integer MI_PROC_EXPORT
mi_event_pending ARGS((MI_CONNECTION MI_FAR *miconn));

void MI_PROC_EXPORT
mi_exc_handled ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_exec ARGS((MI_CONNECTION MI_FAR *conn,
	      const MI_STRING MI_FAR *query,
	      mi_integer control));

mi_integer MI_PROC_EXPORT
mi_file_allocate ARGS((mi_integer n));

void MI_PROC_EXPORT
mi_file_close ARGS((mi_integer fd));

mi_integer MI_PROC_EXPORT
mi_file_open ARGS((const char MI_FAR *name,
		   mi_integer flags,
		   mi_integer mode));

mi_integer MI_PROC_EXPORT
mi_file_read ARGS((mi_integer fd,
		   char MI_FAR *buf,
		   mi_integer amnt));

mi_integer MI_PROC_EXPORT
mi_file_seek ARGS((mi_integer fd,
		   mi_integer offset,
		   mi_integer whence));

mi_integer MI_PROC_EXPORT
mi_file_sync ARGS((mi_integer fd));

mi_integer MI_PROC_EXPORT
mi_file_tell ARGS((mi_integer fd));

mi_integer MI_PROC_EXPORT
mi_file_unlink ARGS((mi_integer fd));

mi_integer MI_PROC_EXPORT
mi_file_write ARGS((mi_integer fd,
		    const char MI_FAR *buf,
		    mi_integer amnt));

mi_unsigned_integer MI_PROC_EXPORT
mi_fix_integer ARGS((mi_unsigned_integer val));

mi_oid MI_FAR * MI_PROC_EXPORT
mi_fix_oid ARGS((mi_oid MI_FAR *src,
		 mi_oid MI_FAR *dest));

mi_unsigned_integer MI_PROC_EXPORT
mi_fix_smallint ARGS((mi_unsigned_integer val));

void MI_PROC_EXPORT
mi_free ARGS((char MI_FAR *ptr));

char MI_FAR * MI_PROC_EXPORT
mi_get_application_password ARGS((void));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_get_bytes ARGS((unsigned char MI_FAR *iodata,
		   char MI_FAR *valp,
		   mi_integer nbytes));

mi_integer MI_PROC_EXPORT
mi_get_callback ARGS((MI_EVENT_TYPE event_type,
		      MI_VOID *ret_function,
		      void MI_FAR * MI_FAR *ret_user_data));

mi_boolean MI_PROC_EXPORT
mi_get_client_home ARGS((char MI_FAR *client_home_path,
			 mi_integer client_home_path_length));

mi_integer MI_PROC_EXPORT
mi_get_connection_info ARGS((MI_CONNECTION MI_FAR *conn,
			     MI_CONNECTION_INFO MI_FAR *ret));

mi_integer MI_PROC_EXPORT
mi_get_connection_user_data ARGS((MI_CONNECTION MI_FAR *conn,
				  void MI_FAR * MI_FAR *user_data));

mi_integer MI_PROC_EXPORT
mi_get_database_info ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_DATABASE_INFO MI_FAR *ret));

mi_integer MI_PROC_EXPORT
mi_get_default_connection_info ARGS((MI_CONNECTION_INFO MI_FAR *ret));

mi_integer MI_PROC_EXPORT
mi_get_default_database_info ARGS((MI_DATABASE_INFO MI_FAR *ret));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_get_double_precision ARGS((unsigned char MI_FAR *datap,
			      mi_double_precision MI_FAR *valp));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_get_integer ARGS((unsigned char MI_FAR *datap,
		     mi_integer MI_FAR *mi_integerp));

mi_integer MI_PROC_EXPORT
mi_get_license ARGS((MI_CONNECTION MI_FAR *conn,
		     const char MI_FAR *feature));

mi_boolean MI_PROC_EXPORT
mi_get_next_sysalias ARGS((const char MI_FAR *sysname,
			   mi_integer MI_FAR *hiddenSysParam,
			   char MI_FAR *alias_name,
			   mi_integer alias_length));

mi_boolean MI_PROC_EXPORT
mi_get_next_sysname ARGS((mi_integer MI_FAR *hiddenSysList,
			  char MI_FAR *system_name,
			  mi_integer length));

mi_boolean MI_PROC_EXPORT
mi_get_next_sysparam ARGS((const char MI_FAR *sysname,
			   mi_integer MI_FAR *hiddenSysParam,
			   char MI_FAR *param_name,
			   mi_integer param_length,
			   char MI_FAR *value,
			   mi_integer value_length));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_get_oid ARGS((unsigned char MI_FAR *datap,
		 mi_oid MI_FAR *oidp));

mi_integer MI_PROC_EXPORT
mi_get_parameter_info ARGS((MI_PARAMETER_INFO MI_FAR *ret));

mi_integer MI_PROC_EXPORT
mi_get_result ARGS((MI_CONNECTION MI_FAR *conn));

MI_ROW_DESC MI_FAR * MI_PROC_EXPORT
mi_get_row_desc ARGS((MI_ROW MI_FAR *row));

MI_ROW_DESC MI_FAR * MI_PROC_EXPORT
mi_get_row_desc_without_row ARGS((MI_CONNECTION MI_FAR *conn));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_get_smallint ARGS((unsigned char MI_FAR *datap,
		      mi_smallint MI_FAR *smallintp));

char MI_FAR * MI_PROC_EXPORT
mi_getsysparam ARGS((const char MI_FAR *sysname,
		     const char MI_FAR *name));

mi_integer MI_PROC_EXPORT
mi_init_library ARGS((void));

void MI_PROC_EXPORT
mi_interrupt_check ARGS((void));

mi_integer MI_PROC_EXPORT
mi_library_version ARGS((char MI_FAR *buf,
			 mi_integer buflen));

mi_integer MI_PROC_EXPORT
mi_login ARGS((MI_CONNECTION MI_FAR *conn,
	       const MI_DATABASE_INFO MI_FAR *dbinfo));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_next_row ARGS((MI_CONNECTION MI_FAR *conn,
		  mi_integer MI_FAR *error));

mi_integer MI_PROC_EXPORT
mi_oid_to_buffer ARGS((const mi_oid MI_FAR *oid, char MI_FAR *buf));

char MI_FAR * MI_PROC_EXPORT
mi_oid_to_string ARGS((const mi_oid MI_FAR *oid));

MI_CONNECTION MI_FAR * MI_PROC_EXPORT
mi_open ARGS((const char MI_FAR *database_name,
	      const char MI_FAR *user_name,
	      const char MI_FAR *user_passwd));

MI_ASYNC_ACTION MI_PROC_EXPORT
mi_processing_needed ARGS((MI_CONNECTION MI_FAR *miconn,
			   mi_integer flags));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_put_bytes ARGS((unsigned char MI_FAR *iodata,
		   char MI_FAR *valp,
		   mi_integer nbytes));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_put_double_precision ARGS((unsigned char MI_FAR *datap,
			      mi_double_precision MI_FAR *valp));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_put_integer ARGS((unsigned char MI_FAR *datap,
		     mi_integer mi_integerval));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_put_oid ARGS((unsigned char MI_FAR *datap,
		 mi_oid MI_FAR *oidp));

unsigned char MI_FAR * MI_PROC_EXPORT
mi_put_smallint ARGS((unsigned char MI_FAR *datap,
		      mi_integer val));

mi_integer MI_PROC_EXPORT
mi_query_finish ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_query_interrupt ARGS((MI_CONNECTION MI_FAR *conn,
			 mi_integer block_until_acknowledged));

MI_CALLBACK_HANDLE MI_FAR * MI_PROC_EXPORT
mi_register_callback ARGS((MI_CONNECTION *conn,
			   MI_EVENT_TYPE event_type,
			   MI_CALLBACK_FUNC func,
			   void MI_FAR *user_data,
			   MI_CALLBACK_HANDLE MI_FAR *parent));

mi_integer MI_PROC_EXPORT
mi_register_func ARGS((MI_C_FUNC_PTR func,
		       const MI_STRING MI_FAR *fsqlname,
		       mi_integer nargs,
		       MI_STRING MI_FAR * MI_FAR typenamearray[]));

mi_integer MI_PROC_EXPORT
mi_register_function ARGS((MI_CONNECTION MI_FAR *conn,
			   MI_FUNCTYPE type,
			   MI_C_FUNC_PTR func,
			   const MI_STRING MI_FAR *sql_funcname,
			   const MI_STRING MI_FAR *funcname,
			   mi_integer nargs,
			   MI_STRING MI_FAR *typenamearray[]));

mi_integer MI_PROC_EXPORT
mi_register_vb_func ARGS((MI_C_FUNC_PTR func,
			  const MI_STRING MI_FAR *fsqlname,
			  const MI_STRING MI_FAR *ftruename,
			  mi_integer nargs,
			  MI_STRING MI_FAR * MI_FAR typenamearray[]));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_result_command_name ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_result_object_id ARGS((MI_CONNECTION MI_FAR *conn,
			  char MI_FAR *buf,
			  mi_integer buflen));

mi_integer MI_PROC_EXPORT
mi_result_row_count ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_retrieve_callback ARGS((MI_CONNECTION *conn,
			   MI_EVENT_TYPE event_type,
			   MI_CALLBACK_HANDLE MI_FAR *handle,
			   MI_CALLBACK_FUNC *ret_func,
			   void MI_FAR * MI_FAR *ret_user_data));

mi_integer MI_PROC_EXPORT
mi_save_set_count ARGS((MI_SAVE_SET MI_FAR *save_set));

MI_SAVE_SET MI_FAR * MI_PROC_EXPORT
mi_save_set_create ARGS((MI_CONNECTION MI_FAR *conn));

mi_integer MI_PROC_EXPORT
mi_save_set_delete ARGS((MI_ROW MI_FAR *row));

mi_integer MI_PROC_EXPORT
mi_save_set_destroy ARGS((MI_SAVE_SET MI_FAR *save_set));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_save_set_get_first ARGS((MI_SAVE_SET MI_FAR *save_set,
			    mi_integer MI_FAR *error));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_save_set_get_last ARGS((MI_SAVE_SET MI_FAR *save_set,
			   mi_integer MI_FAR *error));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_save_set_get_next ARGS((MI_SAVE_SET MI_FAR *save_set,
			   mi_integer MI_FAR *error));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_save_set_get_previous ARGS((MI_SAVE_SET MI_FAR *save_set,
			       mi_integer MI_FAR *error));

MI_ROW MI_FAR * MI_PROC_EXPORT
mi_save_set_insert ARGS((MI_SAVE_SET MI_FAR *save_set,
			 MI_ROW MI_FAR *row));

mi_integer MI_PROC_EXPORT
mi_save_set_member ARGS((MI_ROW MI_FAR *row));

MI_CONNECTION MI_FAR * MI_PROC_EXPORT
mi_server_connect ARGS((MI_CONNECTION_INFO MI_FAR *conn_info));

mi_integer MI_PROC_EXPORT
mi_server_reconnect ARGS((MI_CONNECTION MI_FAR *conn));

void MI_PROC_EXPORT
mi_set_application_password ARGS((char MI_FAR *app_pw));

mi_integer MI_PROC_EXPORT
mi_set_connection_user_data ARGS((MI_CONNECTION MI_FAR *conn,
				  void MI_FAR *user_data));

mi_integer MI_PROC_EXPORT
mi_set_default_connection_info ARGS((const MI_CONNECTION_INFO MI_FAR *cinfo));

mi_integer MI_PROC_EXPORT
mi_set_default_database_info ARGS((const MI_DATABASE_INFO MI_FAR *dinfo));

void MI_PROC_EXPORT
mi_set_encrypt_function ARGS((MI_VARLENA_FUNC_PTR func,
			      void *arg));

mi_integer MI_PROC_EXPORT
mi_set_parameter_info ARGS((const MI_PARAMETER_INFO MI_FAR *set));

mi_integer MI_PROC_EXPORT
mi_setsysparam ARGS((const char MI_FAR *sysname,
		     const char MI_FAR *name,
		     const char MI_FAR *value));

mi_oid MI_FAR * MI_PROC_EXPORT
mi_string_to_oid ARGS((const char MI_FAR *str));

mi_integer MI_PROC_EXPORT
mi_string_to_oidbuffer ARGS((const char MI_FAR *str, mi_oid MI_FAR *oid));

mi_systext MI_FAR * MI_PROC_EXPORT
mi_string_to_systext ARGS((MI_CONNECTION MI_FAR *conn,
			   const MI_STRING MI_FAR *str));

mi_text MI_FAR * MI_PROC_EXPORT
mi_string_to_text ARGS((const char MI_FAR *str));

MI_MEMORY_DURATION MI_PROC_EXPORT
mi_switch_mem_duration ARGS((MI_MEMORY_DURATION duration));

char MI_FAR * MI_PROC_EXPORT
mi_sysname ARGS((const char MI_FAR *sysname));

MI_STRING MI_FAR * MI_PROC_EXPORT
mi_systext_to_string ARGS((MI_CONNECTION MI_FAR *conn,
			   const mi_systext MI_FAR *txt));

mi_integer MI_PROC_EXPORT
mi_tape_level ARGS((void MI_FAR *tr));

mi_integer MI_PROC_EXPORT
mi_tape_reply ARGS((MI_CONNECTION MI_FAR *conn,
		    const char MI_FAR *reply));

mi_integer MI_PROC_EXPORT
mi_tape_volume ARGS((void MI_FAR *tr));

void MI_PROC_EXPORT
mi_text_to_buffer ARGS((const mi_text MI_FAR *lena,
			char MI_FAR *buf));

char MI_FAR * MI_PROC_EXPORT
mi_text_to_string ARGS((const mi_text MI_FAR *lena));

mi_integer MI_PROC_EXPORT
mi_unix_connection ARGS((MI_CONNECTION MI_FAR *miconn));

mi_integer MI_PROC_EXPORT
mi_unregister_callback ARGS((MI_CONNECTION *conn,
			     MI_EVENT_TYPE event_type,
			     MI_CALLBACK_HANDLE MI_FAR *handle));

mi_integer MI_PROC_EXPORT
mi_value ARGS((MI_ROW MI_FAR *row,
	       mi_integer value_no,
	       char MI_FAR * MI_FAR *retbuf,
	       mi_integer MI_FAR *retlen));

mi_integer MI_PROC_EXPORT
mi_value_length ARGS((MI_ROW MI_FAR *row,
		      mi_integer value_no));

mi_varlena MI_FAR * MI_PROC_EXPORT
mi_varlena_copy ARGS((const mi_varlena MI_FAR *t));

void MI_PROC_EXPORT
mi_xact_levels ARGS((void MI_FAR *xe,
		     mi_integer MI_FAR *oldlevel,
		     mi_integer MI_FAR *newlevel));

mi_integer MI_PROC_EXPORT
mi_xact_state ARGS((void MI_FAR *xe));

char MI_FAR * MI_PROC_EXPORT
mi_zalloc ARGS((mi_integer len));


EXTERNC_END

#endif /* _MILIB_H_ */
